<?php

/**
 * @file
 * Controller routines for Juicebox XML.
 */

namespace Drupal\juicebox\Controller;

use Symfony\Component\HttpKernel\Exception\AccessDeniedHttpException;
use Symfony\Component\HttpKernel\Exception\NotFoundHttpException;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpFoundation\RequestStack;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Cache\CacheBackendInterface;
use Drupal\juicebox\JuiceboxGalleryInterface;

/**
 * Controller routines for Juicebox XML.
 */
abstract class JuiceboxXmlControllerBase implements ContainerInjectionInterface {

  protected $configFactory;
  protected $request;
  protected $juicebox_cache;
  protected $settings = array();

  /**
   * Constructor
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The Drupal config factory that can be used to derive global Juicebox
   *   settings.
   * @param \Symfony\Component\HttpFoundation\RequestStack $request_stack
   *   The Symfony request stack from which to extract the current request.
   * @param \Drupal\Core\Cache\CacheBackendInterface $juicebox_cache
   *   The cache interface for the Juicebox cache bin.
   */
  public function __construct(ConfigFactoryInterface $config_factory, RequestStack $request_stack, CacheBackendInterface $juicebox_cache) {
    $this->configFactory = $config_factory;
    // Fetch and store the Juicebox-specific global settings.
    $this->settings = $config_factory->get('juicebox.settings')->get();
    $this->request = $request_stack->getCurrentRequest();
    $this->juicebox_cache = $juicebox_cache;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    // Create the actual controller instance.
    return new static($container->get('config.factory'), $container->get('request_stack'), $container->get('cache.juicebox'));
  }

  /**
   * Common controller for the Juicebox XML.
   *
   * @param string $args
   *   The arguments passed from the router.
   * @return Response $xml
   *   A Symfony response object containing the XML information.
   */
  public function xmlController($args = '') {
    // Try to build the XML.
    try {
      // Initialize data and test access.
      $this->init($args);
      if (!$this->access()) {
        throw new AccessDeniedHttpException();
      }
      // We first try to get the gallery from the cache. We run this check even
      // if the Juicebox XML cache feature is disabled as there are some cases
      // where XML generation will always be dependent on the cache (and we
      // can't identify those cases until testing the cache).
      $query = $this->request->query->all();
      if (isset($query['ggid'])) {
        $cached = $this->juicebox_cache->get($query['ggid'], TRUE);
        // If we have cached data we only use it if the the Juicebox cache
        // feature is enabled, or if it's a permanent entry (which indicates
        // that the cache is the only reliable source for the gallery XML).
        if ($cached && ($this->settings['use_xml_cache'] || $cached->expire == CacheBackendInterface::CACHE_PERMANENT) && ($cached->data instanceof JuiceboxGalleryInterface)) {
          $gallery = $cached->data;
        }
      }
      // If we couldn't get anything from the cache we have to try and re-build
      // the gallery. Note that this may fail in certain circumstances (such as
      // pseudo field instances that are considered cache dependent).
      if (!isset($gallery)) {
        $gallery = $this->getGallery();
      }
      // Calculate headers.
      $headers = array(
        'Content-Type' => 'application/xml; charset=utf-8',
        'X-Robots-Tag' => 'noindex',
      );
      if ($this->settings['enable_cors']) {
        $headers['Access-Control-Allow-Origin'] = '*';
      }
      // Also capture relevant Drupal cache tags that may aid in cache cleanup.
      // @todo: Consider if we need to enforce any specific cache expiration
      // info here. It looks like core page caching for anon users will cache
      // the page permanently which won't be good unless our cache tags are
      // very efficient at triggering proper expiration.
      $headers['X-Drupal-Cache-Tags'] = 'juicebox_gallery juicebox_gallery_id:' . $gallery->getId();
      // Package the XML as a Response object.
      return new Response($gallery->renderXml(), 200, $headers);
    }
    catch (\Exception $e) {
      // An access denied exception should just be re-thrown.
      if ($e instanceof AccessDeniedHttpException) {
        throw $e;
      }
      // Otherwise we have an error. Log it and trigger a general 404 response.
      $message = 'Exception building Juicebox XML: !message in %function (line %line of %file).';
      watchdog_exception('juicebox', $e, $message);
      throw new NotFoundHttpException();
    }
  }

  /**
   * Initialize the controller based on the passed arg data.
   *
   * @param string $args
   *   The arguments passed from the router.
   */
  abstract protected function init($args);

  /**
   * Check access to the Drupal data that will be used to build the gallery.
   *
   * @return boolean
   *   Returns TRUE if access is allowed for the current user and FALSE if not.
   *   Can also return NULL if access cannot be determined.
   */
  abstract protected function access();

  /**
   * Get the Juicebox gallery object.
   *
   * @return Drupal\juicebox\JuiceboxGalleryInterface
   *   A Juicebox gallery object.
   */
  abstract protected function getGallery();

}
